﻿package MT::Plugin::Protection;
use strict;
use MT;
use MT::Plugin;
use MT::Log;
use MT::Util qw( offset_time_list );

our $VERSION = '0.5';

use base qw( MT::Plugin );

@MT::Plugin::Protection::ISA = qw( MT::Plugin );

my $plugin = new MT::Plugin::Protection( {
    name => 'Protection',
    description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    version     => $VERSION,
    l10n_class  => 'Protection::L10N',
    settings => new MT::PluginSettings( [
        [ 'period', { Default => 3600 } ],
        [ 'count' , { Default => 3 } ],
        [ 'white_ip' , { Default => '' } ],
        [ 'black_ip' , { Default => '' } ],
    ] ),
    system_config_template => 'config.tmpl',
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        init_request => {
            'MT::App::CMS'
                 => \&init_cms_app_routine,
        },
        callbacks => {
            'MT::App::CMS::template_param.login'
                 => \&_login_param,
        },
   } );
}

sub init_cms_app_routine {
    my $app = shift;
    my $mode = $app->mode;
    my $period = $plugin->get_config_value( 'period' );
    my $count  = $plugin->get_config_value( 'count' );
    my $white_ip  = $plugin->get_config_value( 'white_ip' );
    my $black_ip  = $plugin->get_config_value( 'black_ip' );
    my $remote_ip = $app->remote_ip;
    my $invalid_login;
    if ( $mode ne 'logout' ) {
        if ( $white_ip ) {
            $white_ip=~ s/\r\n/\n/gs;
            $white_ip=~ s/\r/\n/g;
            my @table = split( /\n/, $white_ip );
            if ( ! _valid_ip( $remote_ip, \@table ) ) {
                $invalid_login = 1;
            }
        }
        if ( $black_ip ) {
            $black_ip=~ s/\r\n/\n/gs;
            $black_ip=~ s/\r/\n/g;
            my @table = split( /\n/, $black_ip );
            if ( _valid_ip( $remote_ip, \@table ) ) {
                $invalid_login = 1;
            }
        }
        if (! $invalid_login ) {
            if ( $period && $count ) {
                my $time = time - ( $app->config->TimeOffset ) * 3600;
                my @tl = offset_time_list( $time - $period, undef );
                my $days_from = sprintf "%04d%02d%02d%02d%02d%02d", $tl[5]+1900, $tl[4]+1, @tl[3,2,1,0];
                my $bad_login = MT::Log->count( { category  => 'login_user', ip => $remote_ip },
                                                { sort      => 'modified_on',
                                                  start_val => $days_from,
                                                  direction => 'ascend',
                                                 } );
                if ( $bad_login >= $count ) {
                    $invalid_login = 1;
                }
            }
        }
    }
    if ( $invalid_login ) {
        my $username = $app->param( 'username' );
        $username = 'Unknown' unless $username;
        MT->log( {
            message  => $app->translate( 'Invalid login attempt from user \'[_1]\'', $username ),
            ip       => $remote_ip,
            category => 'login_user',
            level    => 2,
            class    => 'system',
        } );
        my $location = $app->base;
        $location .= $app->uri( mode => 'logout', args => { 'bad_login' => 1 } );
        $app->redirect( $location );
    }
}

sub _login_param {
    my ( $cb, $app, $param ) = @_;
    my $bad_login = $app->param( 'bad_login' );
    my $remote_ip = $app->remote_ip;
    if ( $bad_login ) {
        $param->{ error } = $app->translate( 'Invalid login.' );
        $param->{ logged_out } = 0;
    } else {
        my $error = $param->{ error };
        my $error_msg = $app->translate( 'Invalid login.' );
        if ( $error =~ m/^$error_msg/ ) {
            my $username = $app->param( 'username' );
            if ( $username ) {
                my $author = MT::Author->load( { name => $username } );
                if ( defined $author ) {
                    my $password = $app->param( 'password' );
                    unless ( $author->is_valid_password( $password ) ) {
                        MT->log( {
                            message  => $app->translate( 'Login failed: password was wrong for user \'[_1]\'', $username ),
                            ip       => $remote_ip,
                            category => 'login_user',
                            level    => 2,
                            class    => 'system',
                        } );
                    }
                }
            } else {
                MT->log( {
                    message  => $app->translate( 'Invalid login attempt from user \'[_1]\'', 'unknown' ),
                    ip       => $remote_ip,
                    category => 'login_user',
                    level    => 2,
                    class    => 'system',
                } );
            }
        }
    }
}

sub _valid_ip {
    my ( $remote_ip, $table ) = @_;
    return 1 if ( grep( /^$remote_ip$/, @$table ) );
    my $ip_table = join ( "\n", @$table );
    if ( $remote_ip =~ /(^[0-9]{1,}\.[0-9]{1,}\.[0-9]{1,}\.)([0-9]{1,}$)/ ) {
        my @bits = qw/ 0 126 62 30 14 6 2 /;
        my $check = quotemeta( $1 );
        my $last = $2;
        if ( $ip_table =~ /$check([0-9]{1,})\/([0-9]{1,})/ ) {
            my $begin = $1;
            my $bit = $2;
            if ( ( $begin eq '0' ) && ( $bit eq '24' ) ) {
                if ( ( 0 < $last ) && ( 255 > $last ) ) {
                    return 1;
                }
            } else {
                $bit = $bit - 24;
                my $range = $bits[ $bit ];
                my $end = $begin + $range;
                if ( ( $last >= $begin ) && ( $last <= $end ) ) {
                    return 1;
                }
            }
        }
    }
    return 0;
}

1;